pub type DynTensorRef<'v> = ValueRef<'v, DynTensorValueType>;Expand description
A reference to a tensor Value whose data type is unknown.
Aliased Type
pub struct DynTensorRef<'v> { /* private fields */ }pub type DynTensorRef<'v> = ValueRef<'v, DynTensorValueType>;A reference to a tensor Value whose data type is unknown.
pub struct DynTensorRef<'v> { /* private fields */ }